
#include "stdafx.h"
#include "Translate.h"
#include "Events.h"

extern int nxtAvailEventId;

#define Str details.Fetch_String
#define Int details.Fetch_Int



class SMALL_TOWN_DATA : public GameEvent
{
public:
  SMALL_TOWN_DATA(void)
  {
    hasTavern=hasTemple=hasVault=hasShop=hasInn=hasTrainingHall=false;
    TempleChain=TrainingHallChain=ShopChain=InnChain=TavernChain=VaultChain=0;
    highestSpellLevel = 0;
    canClericTrain=canFighterTrain=canMageTrain=canPaladinTrain=canRangerTrain=canThiefTrain=false;
    tavernTale.Empty();
    shopCostModifier = trainingCostModifier = 0;
    items.RemoveAll();
  };
  void Export(JWriter& jw);
  bool Read(JReader& event, JReader& jr);
  long Unused;
  DWORD TempleChain;
  DWORD TrainingHallChain;
  DWORD ShopChain;
  DWORD InnChain;
  DWORD TavernChain;
  DWORD VaultChain;

  CArray<CString, CString&> items;
  CString tavernTale;
  int  highestSpellLevel;
  int  trainingCostModifier;
  int  shopCostModifier;
  bool hasTavern;
  bool hasTemple;
  bool hasVault;
  bool hasShop;
  bool hasInn;
  bool hasTrainingHall;
  bool canClericTrain; 
  bool canFighterTrain; 
  bool canMageTrain; 
  bool canPaladinTrain; 
  bool canRangerTrain; 
  bool canThiefTrain; 

  void CreateTavern(JWriter& jw, int lineNum);
  void CreateTemple(JWriter& jw, int lineNum);
  void CreateVault(JWriter& jw, int lineNum);
  void CreateInn(JWriter& jw, int lineNum);
  void CreateShop(JWriter& jw, int lineNum);
  void CreateTrainingHall(JWriter& jw, int lineNum);


};


void SMALL_TOWN_DATA::CreateTavern(JWriter& jw, int lineNum)
{
  if (hasTavern)
  {
    TAVERN tavern;
    tavern.jsonLineNum = lineNum;
    tavern.id = nxtAvailEventId++;
    tavern.control.eventTrigger = AlwaysTrigger;
    tavern.tales[0].tale = tavernTale;
    tavern.tales[0].count= 999;
    TavernChain = tavern.id;
    tavern.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}

void SMALL_TOWN_DATA::CreateTemple(JWriter& jw, int lineNum)
{
  if (hasTemple)
  {
    TEMPLE temple;
    temple.jsonLineNum = lineNum;
    temple.id = nxtAvailEventId++;
    TempleChain = temple.id;
    temple.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}

void SMALL_TOWN_DATA::CreateVault(JWriter& jw, int lineNum)
{
  if (hasVault)
  {
    VAULT_EVENT_DATA Vault;
    Vault.jsonLineNum = lineNum;
    Vault.id = nxtAvailEventId++;
    VaultChain = Vault.id;
    Vault.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}

void SMALL_TOWN_DATA::CreateInn(JWriter& jw, int lineNum)
{
  if (hasInn)
  {
    CAMP_EVENT_DATA camp;
    camp.jsonLineNum = lineNum;
    camp.id = nxtAvailEventId++;
    InnChain = camp.id;
    camp.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}

void SMALL_TOWN_DATA::CreateShop(JWriter& jw, int lineNum)
{
  if (hasShop)
  {
    SHOP shop;
    shop.jsonLineNum = lineNum;
    shop.id = nxtAvailEventId++;
    ShopChain = shop.id;
    shop.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}

void SMALL_TOWN_DATA::CreateTrainingHall(JWriter& jw, int lineNum)
{
  if (hasTrainingHall)
  {
    TRAININGHALL trainingHall;
    trainingHall.jsonLineNum = lineNum;
    trainingHall.id = nxtAvailEventId++;
    TrainingHallChain = trainingHall.id;
    trainingHall.Export(jw);
    jw.EndList();
    jw.NextEntry();
    jw.StartList();
  };
}


void Event_SmallTown(JReader& event, JReader& details, JWriter& jw)
{
  SMALL_TOWN_DATA small_town_data;
  small_town_data.Read(event, details);

  small_town_data.CreateTavern(jw, small_town_data.jsonLineNum);
  small_town_data.CreateTemple(jw, small_town_data.jsonLineNum);
  small_town_data.CreateTrainingHall(jw, small_town_data.jsonLineNum);
  small_town_data.CreateShop(jw, small_town_data.jsonLineNum);
  small_town_data.CreateInn(jw, small_town_data.jsonLineNum);
  small_town_data.CreateVault(jw, small_town_data.jsonLineNum);

  small_town_data.Export(jw);
}



void SMALL_TOWN_DATA::Export(JWriter& jw)
{
  GameEvent::Export(jw);
  jw.StartList(JKEY_PARTICULAR);
  jw.StartList(JKEY_CHAINS);
  jw.Linefeed(false);

  jw.NameAndNonZero(JKEY_TEMPLE, TempleChain);
  jw.NameAndNonZero(JKEY_TRAIN, TrainingHallChain);
  jw.NameAndNonZero(JKEY_SHOP, ShopChain);
  jw.NameAndNonZero(JKEY_INN, InnChain);
  jw.NameAndNonZero(JKEY_TAVERN, TavernChain);
  jw.NameAndNonZero(JKEY_VAULT, VaultChain);
  jw.EndList();
  jw.Linefeed(true);
  jw.EndList();
}

bool SMALL_TOWN_DATA::Read(JReader& event, JReader& details)
{
  GameEvent::Read(event, SmallTown);

  details.NameAndBool("has_inn",hasInn);
  details.NameAndBool("has_shop",hasShop);
  details.NameAndBool("has_tavern",hasTavern);
  details.NameAndBool("has_temple" ,hasTemple);
  details.NameAndBool("has_traininghall",hasTrainingHall);
  details.NameAndBool("has_vault",hasVault);
 

  details.NameAndValue("text", text);
  details.NameAndValue("tavern_tale", tavernTale);
  details.NameAndValue("picture", pic.filename);
  details.NameAndValue("traininghall_temple_cost_modifier", trainingCostModifier);
  details.NameAndValue("shop_cost_modifier", shopCostModifier);
  details.NameAndBool("can_cleric_train", canClericTrain);
  details.NameAndBool("can_fighter_train", canFighterTrain);
  details.NameAndBool("can_mage_train", canMageTrain);
  details.NameAndBool("can_paladin_train", canPaladinTrain);
  details.NameAndBool("can_ranger_train", canRangerTrain);
  details.NameAndBool("can_thief_train", canThiefTrain);

  details.EndList(DBGID_smallTown);
  return true;
}
